package model.biz;

/**
 * FullName is a simple class that models
 * a full name (first, middle, last) 
 * 
 * Example Usage:
 *
 * 	FullName name1 = FullName.fromFirstMiddleLast("Asha", "Devi", "Gupta"); 
 * 	prn(name1.toString());
 * 		Asha Devi Gupta
 * 
 * 	FullName name2 = FullName.fromFirstLast("Asha", "Gupta"); 
 * 	prn(name2.toString());
 * 		Asha Gupta * 
 * */
public class FullName {

	//Each of these three defaults to empty string
	private String first;	 
	private String middle;
	private String last;	
	
	//------------------------------------------
	// Constructing - public	
	
	/** Returns new FullName object setting all components from params */
	public static FullName fromFirstMiddleLast(String f, String m, String l) {
		return new FullName(f, m, l);
	}	
	
	/** Returns new FullName object setting two components from params
	 * (middle name will be empty string) */
	public static FullName fromFirstLast(String f, String l) {
		return fromFirstMiddleLast(f, "", l);
	}	
	
	/** Returns new FullName with all name components blank */
	public static FullName newBlank() {
		return fromFirstMiddleLast("", "", "");
	}	
	
	//------------------------------------------
	// Constructing - private

	/** Constructs FullName with passed lat and long values
	 * e.g., new FullName(45.076076, -93.332728) */
	private FullName(String f, String m, String l) {
		this.first = f;
		this.middle = m;
		this.last = l;
	}
	
	//------------------------------------------
	// Calculating
	
	/** Returns true if name is valid (has non-blank first and last name) */
	public boolean isValid() {
		return !this.first.isBlank() && !this.last.isBlank(); 
	}
	
	/** Returns name reversed -- last + middle + first (space delimited) */
	public String toReversed() {
		return this.join(this.last, this.middle, this.first);
	}	
	
	/** Returns true this full name equals passed full name (param) */
	public boolean matches(FullName other) {
		return 
			this.first.equals(other.first)
				&& this.middle.equals(other.middle)
				&& this.last.equals(other.last);
	}	
	
	/** Returns true this customer has same last name as passed full name (param) */
	public boolean matchesLastNameOf(FullName other) {
		return this.last.equals(other.last);
	}	
	
	//------------------------------------------
	// Common

	/** Returns "nice string" of this FullName */
	public String toString() {
		return this.join(this.first, this.middle, this.last);		
	}
	
	//------------------------------------------
	// Private Helpers
	
	private String join(String a, String b, String c) {
		//Be careful not to end up with double-spaces
		StringBuilder sb = new StringBuilder();
		if (!a.isBlank()) sb.append(a);
		if (!b.isBlank()) sb.append(" " + b);
		if (!c.isBlank()) sb.append(" " + c);
		return sb.toString().trim();
	}

}